@@ -146,8 +146,14 @@ async fn get_latest_version(duration: Duration) -> Option<String> {
     let version_file_path = dirs::CACHE.join("latest-version");
     if let Ok(metadata) = modified_duration(&version_file_path) {
         if metadata < duration {
-            if let Ok(version) = file::read_to_string(&version_file_path) {
-                return Some(version.trim().to_string());
+            if let Some(version) = file::read_to_string(&version_file_path)
+                .ok()
+                .map(|s| s.trim().to_string())
+                .and_then(Versioning::new)
+            {
+                if *V <= version {
+                    return Some(version.to_string());
+                }
             }
         }
     }
