@@ -71,8 +71,8 @@ mod tests {
             @r#"
         [
             "~/foo/1",
-            "~/cwd/rel2/2",
-            "~/cwd/rel/1",
+            "~/rel2/2",
+            "~/rel/1",
             "/path/1",
             "/path/2",
         ]
